@@ -57,8 +57,8 @@ extern "C" HINSTANCE s_hModule;           // Saved module handle
 extern char *JvmPath;   // The connect_jvm_path global variable value
 extern char *ClassPath; // The connect_class_path global variable value
 
-char *GetPluginDir(void);
 char *GetJavaWrapper(void);		// The connect_java_wrapper variable value
+extern MYSQL_PLUGIN_IMPORT char lc_messages_dir[FN_REFLEN];
 
 /***********************************************************************/
 /*  Static JAVAConn objects.                                           */
@@ -401,23 +401,23 @@ bool JAVAConn::Open(PGLOBAL g)
 		}	// endif ClassPath
 
 #if 0
-		// Java source will be compiled as a jar file installed in the plugin dir
+		// Java source will be compiled as a jar file installed in the mysql share dir
 		jpop->Append(sep);
-		jpop->Append(GetPluginDir());
+		jpop->Append(lc_messages_dir);
 		jpop->Append("JdbcInterface.jar");
 #endif // 0
 
-		// All wrappers are pre-compiled in JavaWrappers.jar in the plugin dir
+		// All wrappers are pre-compiled in JavaWrappers.jar in the mysql share dir
 		jpop->Append(sep);
-		jpop->Append(GetPluginDir());
+		jpop->Append(lc_messages_dir);
 		jpop->Append("JavaWrappers.jar");
 
 #if defined(MONGO_SUPPORT)
 		jpop->Append(sep);
-		jpop->Append(GetPluginDir());
+		jpop->Append(lc_messages_dir);
 		jpop->Append("Mongo3.jar");
 		jpop->Append(sep);
-		jpop->Append(GetPluginDir());
+		jpop->Append(lc_messages_dir);
 		jpop->Append("Mongo2.jar");
 #endif   // MONGO_SUPPORT
 
