@@ -92,7 +92,7 @@ inline bool TrxUndoRsegsIterator::set_next()
 		number shouldn't increase. Undo the increment of
 		expected commit done by caller assuming rollback
 		segments from given transaction are done. */
-		purge_sys.tail.commit = (*m_iter)->last_commit;
+		purge_sys.tail.trx_no = (*m_iter)->last_trx_no();
 	} else if (!purge_sys.purge_queue.empty()) {
 		m_rsegs = purge_sys.purge_queue.top();
 		purge_sys.purge_queue.pop();
@@ -110,24 +110,23 @@ inline bool TrxUndoRsegsIterator::set_next()
 
 	purge_sys.rseg = *m_iter++;
 	mysql_mutex_unlock(&purge_sys.pq_mutex);
-	mysql_mutex_lock(&purge_sys.rseg->mutex);
+	purge_sys.rseg->latch.rd_lock();
 
 	ut_a(purge_sys.rseg->last_page_no != FIL_NULL);
-	ut_ad(purge_sys.rseg->last_trx_no() == m_rsegs.trx_no());
+	ut_ad(purge_sys.rseg->last_trx_no() == m_rsegs.trx_no);
 
 	/* We assume in purge of externally stored fields that space id is
 	in the range of UNDO tablespace space ids */
 	ut_ad(purge_sys.rseg->space->id == TRX_SYS_SPACE
 	      || srv_is_undo_tablespace(purge_sys.rseg->space->id));
 
-	ut_a(purge_sys.tail.commit <= purge_sys.rseg->last_commit);
+	ut_a(purge_sys.tail.trx_no <= purge_sys.rseg->last_trx_no());
 
-	purge_sys.tail.commit = purge_sys.rseg->last_commit;
-	purge_sys.hdr_offset = purge_sys.rseg->last_offset;
+	purge_sys.tail.trx_no = purge_sys.rseg->last_trx_no();
+	purge_sys.hdr_offset = purge_sys.rseg->last_offset();
 	purge_sys.hdr_page_no = purge_sys.rseg->last_page_no;
 
-	mysql_mutex_unlock(&purge_sys.rseg->mutex);
-
+	purge_sys.rseg->latch.rd_unlock();
 	return(true);
 }
 
@@ -214,8 +213,7 @@ trx_purge_add_undo_to_history(const trx_t* trx, trx_undo_t*& undo, mtr_t* mtr)
 {
 	DBUG_PRINT("trx", ("commit(" TRX_ID_FMT "," TRX_ID_FMT ")",
 			   trx->id, trx_id_t{trx->rw_trx_hash_element->no}));
-	ut_ad(undo == trx->rsegs.m_redo.undo
-	      || undo == trx->rsegs.m_redo.old_insert);
+	ut_ad(undo == trx->rsegs.m_redo.undo);
 	trx_rseg_t*	rseg		= trx->rsegs.m_redo.rseg;
 	ut_ad(undo->rseg == rseg);
 	buf_block_t*	rseg_header	= trx_rsegf_get(
@@ -282,12 +280,11 @@ trx_purge_add_undo_to_history(const trx_t* trx, trx_undo_t*& undo, mtr_t* mtr)
 		  && (srv_is_being_started
 		      || trx_rollback_is_active
 		      || srv_force_recovery >= SRV_FORCE_NO_BACKGROUND))
-	      || ((trx->mysql_thd || trx->internal)
-		  && srv_fast_shutdown));
+	      || srv_fast_shutdown);
 
 #ifdef	WITH_WSREP
-	if (wsrep_is_wsrep_xid(trx->xid)) {
-		trx_rseg_update_wsrep_checkpoint(rseg_header, trx->xid, mtr);
+	if (wsrep_is_wsrep_xid(&trx->xid)) {
+		trx_rseg_update_wsrep_checkpoint(rseg_header, &trx->xid, mtr);
 	}
 #endif
 
@@ -306,23 +303,17 @@ trx_purge_add_undo_to_history(const trx_t* trx, trx_undo_t*& undo, mtr_t* mtr)
 	mtr->write<8,mtr_t::MAYBE_NOP>(*undo_page,
 				       undo_header + TRX_UNDO_TRX_NO,
 				       trx->rw_trx_hash_element->no);
-	/* This is needed for upgrading old undo log pages from
-	before MariaDB 10.3.1. */
-	if (UNIV_UNLIKELY(!mach_read_from_2(undo_header
-					    + TRX_UNDO_NEEDS_PURGE))) {
-		mtr->write<2>(*undo_page, undo_header + TRX_UNDO_NEEDS_PURGE,
-			      1U);
-	}
+	mtr->write<2,mtr_t::MAYBE_NOP>(*undo_page, undo_header
+				       + TRX_UNDO_NEEDS_PURGE, 1U);
 
 	if (rseg->last_page_no == FIL_NULL) {
 		rseg->last_page_no = undo->hdr_page_no;
-		rseg->last_offset = undo->hdr_offset;
-		rseg->set_last_trx_no(trx->rw_trx_hash_element->no,
-				      undo == trx->rsegs.m_redo.undo);
-		rseg->needs_purge = true;
+		rseg->set_last_commit(undo->hdr_offset,
+				      trx->rw_trx_hash_element->no);
+		rseg->set_needs_purge();
 	}
 
-	trx_sys.rseg_history_len++;
+	rseg->history_size++;
 
 	if (undo->state == TRX_UNDO_CACHED) {
 		UT_LIST_ADD_FIRST(rseg->undo_cached, undo);
@@ -345,24 +336,25 @@ static void trx_purge_remove_log_hdr(buf_block_t *rseg, buf_block_t* log,
 {
   flst_remove(rseg, TRX_RSEG + TRX_RSEG_HISTORY,
               log, static_cast<uint16_t>(offset + TRX_UNDO_HISTORY_NODE), mtr);
-  trx_sys.rseg_history_len--;
 }
 
 /** Free an undo log segment, and remove the header from the history list.
 @param[in,out]	rseg		rollback segment
 @param[in]	hdr_addr	file address of log_hdr */
-static
-void
-trx_purge_free_segment(trx_rseg_t* rseg, fil_addr_t hdr_addr)
+static void trx_purge_free_segment(trx_rseg_t *rseg, fil_addr_t hdr_addr)
 {
 	mtr_t		mtr;
 
 	mtr.start();
-	mysql_mutex_lock(&rseg->mutex);
+	const page_id_t hdr_page_id(rseg->space->id, hdr_addr.page);
+
+	/* We only need the latch to maintain rseg->curr_size. To follow the
+	latching order, we must acquire it before acquiring any related
+	page latch.  */
+	rseg->latch.wr_lock();
 
 	buf_block_t* rseg_hdr = trx_rsegf_get(rseg->space, rseg->page_no, &mtr);
-	buf_block_t* block = trx_undo_page_get(
-		page_id_t(rseg->space->id, hdr_addr.page), &mtr);
+	buf_block_t* block = trx_undo_page_get(hdr_page_id, &mtr);
 
 	/* Mark the last undo log totally purged, so that if the
 	system crashes, the tail of the undo log will not get accessed
@@ -375,17 +367,14 @@ trx_purge_free_segment(trx_rseg_t* rseg, fil_addr_t hdr_addr)
 	while (!fseg_free_step_not_header(
 		       TRX_UNDO_SEG_HDR + TRX_UNDO_FSEG_HEADER
 		       + block->frame, &mtr)) {
-		mysql_mutex_unlock(&rseg->mutex);
-
+		rseg->latch.wr_unlock();
 		mtr.commit();
 		mtr.start();
-
-		mysql_mutex_lock(&rseg->mutex);
+		rseg->latch.wr_lock();
 
 		rseg_hdr = trx_rsegf_get(rseg->space, rseg->page_no, &mtr);
 
-		block = trx_undo_page_get(
-			page_id_t(rseg->space->id, hdr_addr.page), &mtr);
+		block = trx_undo_page_get(hdr_page_id, &mtr);
 	}
 
 	/* The page list may now be inconsistent, but the length field
@@ -419,11 +408,12 @@ trx_purge_free_segment(trx_rseg_t* rseg, fil_addr_t hdr_addr)
 
 	ut_ad(rseg->curr_size >= seg_size);
 
+	rseg->history_size--;
 	rseg->curr_size -= seg_size;
 
-	mysql_mutex_unlock(&rseg->mutex);
+	rseg->latch.wr_unlock();
 
-	mtr_commit(&mtr);
+	mtr.commit();
 }
 
 /** Remove unnecessary history data from a rollback segment.
@@ -442,7 +432,7 @@ trx_purge_truncate_rseg_history(
 
 	mtr.start();
 	ut_ad(rseg.is_persistent());
-	mysql_mutex_lock(&rseg.mutex);
+	rseg.latch.wr_lock();
 
 	buf_block_t* rseg_hdr = trx_rsegf_get(rseg.space, rseg.page_no, &mtr);
 
@@ -454,7 +444,7 @@ trx_purge_truncate_rseg_history(
 loop:
 	if (hdr_addr.page == FIL_NULL) {
 func_exit:
-		mysql_mutex_unlock(&rseg.mutex);
+		rseg.latch.wr_unlock();
 		mtr.commit();
 		return;
 	}
@@ -465,8 +455,8 @@ func_exit:
 	undo_trx_no = mach_read_from_8(block->frame + hdr_addr.boffset
 				       + TRX_UNDO_TRX_NO);
 
-	if (undo_trx_no >= limit.trx_no()) {
-		if (undo_trx_no == limit.trx_no()) {
+	if (undo_trx_no >= limit.trx_no) {
+		if (undo_trx_no == limit.trx_no) {
 			trx_undo_truncate_start(
 				&rseg, hdr_addr.page,
 				hdr_addr.boffset, limit.undo_no);
@@ -487,7 +477,7 @@ func_exit:
 
 		/* We can free the whole log segment */
 
-		mysql_mutex_unlock(&rseg.mutex);
+		rseg.latch.wr_unlock();
 		mtr.commit();
 
 		/* calls the trx_purge_remove_log_hdr()
@@ -497,13 +487,13 @@ func_exit:
 		/* Remove the log hdr from the rseg history. */
 		trx_purge_remove_log_hdr(rseg_hdr, block, hdr_addr.boffset,
 					 &mtr);
-
-		mysql_mutex_unlock(&rseg.mutex);
+		rseg.history_size--;
+		rseg.latch.wr_unlock();
 		mtr.commit();
 	}
 
 	mtr.start();
-	mysql_mutex_lock(&rseg.mutex);
+	rseg.latch.wr_lock();
 
 	rseg_hdr = trx_rsegf_get(rseg.space, rseg.page_no, &mtr);
 
@@ -557,19 +547,18 @@ function is called, the caller must not have any latches on undo log pages!
 static void trx_purge_truncate_history()
 {
 	ut_ad(purge_sys.head <= purge_sys.tail);
-	purge_sys_t::iterator& head = purge_sys.head.commit
+	purge_sys_t::iterator& head = purge_sys.head.trx_no
 		? purge_sys.head : purge_sys.tail;
 
-	if (head.trx_no() >= purge_sys.low_limit_no()) {
+	if (head.trx_no >= purge_sys.low_limit_no()) {
 		/* This is sometimes necessary. TODO: find out why. */
-		head.reset_trx_no(purge_sys.low_limit_no());
+		head.trx_no = purge_sys.low_limit_no();
 		head.undo_no = 0;
 	}
 
-	for (ulint i = 0; i < TRX_SYS_N_RSEGS; ++i) {
-		if (trx_rseg_t* rseg = trx_sys.rseg_array[i]) {
-			ut_ad(rseg->id == i);
-			trx_purge_truncate_rseg_history(*rseg, head);
+	for (auto& rseg : trx_sys.rseg_array) {
+		if (rseg.space) {
+			trx_purge_truncate_rseg_history(rseg, head);
 		}
 	}
 
@@ -615,58 +604,58 @@ static void trx_purge_truncate_history()
 
 		DBUG_LOG("undo", "marking for truncate: " << file->name);
 
-		for (ulint i = 0; i < TRX_SYS_N_RSEGS; ++i) {
-			if (trx_rseg_t* rseg = trx_sys.rseg_array[i]) {
-				ut_ad(rseg->is_persistent());
-				if (rseg->space == &space) {
-					/* Once set, this rseg will
-					not be allocated to subsequent
-					transactions, but we will wait
-					for existing active
-					transactions to finish. */
-					rseg->skip_allocation = true;
-				}
+		for (auto& rseg : trx_sys.rseg_array) {
+			if (rseg.space == &space) {
+				/* Once set, this rseg will
+				not be allocated to subsequent
+				transactions, but we will wait
+				for existing active
+				transactions to finish. */
+				rseg.set_skip_allocation();
 			}
 		}
 
-		for (ulint i = 0; i < TRX_SYS_N_RSEGS; ++i) {
-			trx_rseg_t*	rseg = trx_sys.rseg_array[i];
-			if (!rseg || rseg->space != &space) {
+		for (auto& rseg : trx_sys.rseg_array) {
+			if (rseg.space != &space) {
 				continue;
 			}
-			mysql_mutex_lock(&rseg->mutex);
-			ut_ad(rseg->skip_allocation);
-			if (rseg->trx_ref_count) {
+			ut_ad(rseg.skip_allocation());
+			if (rseg.is_referenced()) {
+				return;
+			}
+			rseg.latch.rd_lock();
+			ut_ad(rseg.skip_allocation());
+			if (rseg.is_referenced()) {
 not_free:
-				mysql_mutex_unlock(&rseg->mutex);
+				rseg.latch.rd_unlock();
 				return;
 			}
 
-			if (rseg->curr_size != 1) {
+			if (rseg.curr_size != 1) {
 				/* Check if all segments are
 				cached and safe to remove. */
 				ulint cached = 0;
 
 				for (trx_undo_t* undo = UT_LIST_GET_FIRST(
-					     rseg->undo_cached);
+					     rseg.undo_cached);
 				     undo;
 				     undo = UT_LIST_GET_NEXT(undo_list,
 							     undo)) {
-					if (head.trx_no() < undo->trx_id) {
+					if (head.trx_no < undo->trx_id) {
 						goto not_free;
 					} else {
 						cached += undo->size;
 					}
 				}
 
-				ut_ad(rseg->curr_size > cached);
+				ut_ad(rseg.curr_size > cached);
 
-				if (rseg->curr_size > cached + 1) {
+				if (rseg.curr_size > cached + 1) {
 					goto not_free;
 				}
 			}
 
-			mysql_mutex_unlock(&rseg->mutex);
+			rseg.latch.rd_unlock();
 		}
 
 		ib::info() << "Truncating " << file->name;
@@ -682,7 +671,7 @@ not_free:
 		mini-transaction commit and the server was killed, then
 		discarding the to-be-trimmed pages without flushing would
 		break crash recovery. So, we cannot avoid the write. */
-		while (buf_flush_dirty_pages(space.id));
+		while (buf_flush_list_space(&space));
 
 		log_free_check();
 
@@ -732,62 +721,22 @@ not_free:
 
 		buf_block_t* sys_header = trx_sysf_get(&mtr);
 
-		for (ulint i = 0; i < TRX_SYS_N_RSEGS; ++i) {
-			trx_rseg_t* rseg = trx_sys.rseg_array[i];
-			if (!rseg || rseg->space != &space) {
+		for (auto& rseg : trx_sys.rseg_array) {
+			if (rseg.space != &space) {
 				continue;
 			}
 
-			ut_ad(rseg->is_persistent());
-			ut_d(const ulint old_page = rseg->page_no);
-
 			buf_block_t* rblock = trx_rseg_header_create(
 				purge_sys.truncate.current,
-				rseg->id, sys_header, &mtr);
+				i, sys_header, &mtr);
 			ut_ad(rblock);
-			rseg->page_no = rblock
-				? rblock->page.id().page_no() : FIL_NULL;
-			ut_ad(old_page == rseg->page_no);
-
-			/* Before re-initialization ensure that we
-			free the existing structure. There can't be
-			any active transactions. */
-			ut_a(UT_LIST_GET_LEN(rseg->undo_list) == 0);
-			ut_a(UT_LIST_GET_LEN(rseg->old_insert_list) == 0);
-
-			trx_undo_t*	next_undo;
-
-			for (trx_undo_t* undo = UT_LIST_GET_FIRST(
-				     rseg->undo_cached);
-			     undo; undo = next_undo) {
-
-				next_undo = UT_LIST_GET_NEXT(undo_list, undo);
-				UT_LIST_REMOVE(rseg->undo_cached, undo);
-				MONITOR_DEC(MONITOR_NUM_UNDO_SLOT_CACHED);
-				ut_free(undo);
-			}
-
-			UT_LIST_INIT(rseg->undo_list,
-				     &trx_undo_t::undo_list);
-			UT_LIST_INIT(rseg->undo_cached,
-				     &trx_undo_t::undo_list);
-			UT_LIST_INIT(rseg->old_insert_list,
-				     &trx_undo_t::undo_list);
-
 			/* These were written by trx_rseg_header_create(). */
 			ut_ad(!mach_read_from_4(TRX_RSEG + TRX_RSEG_FORMAT
 						+ rblock->frame));
 			ut_ad(!mach_read_from_4(TRX_RSEG + TRX_RSEG_HISTORY_SIZE
 						+ rblock->frame));
-
-			/* Initialize the undo log lists according to
-			the rseg header */
-			rseg->curr_size = 1;
-			rseg->trx_ref_count = 0;
-			rseg->last_page_no = FIL_NULL;
-			rseg->last_offset = 0;
-			rseg->last_commit = 0;
-			rseg->needs_purge = false;
+			rseg.reinit(rblock
+				    ? rblock->page.id().page_no() : FIL_NULL);
 		}
 
 		mtr.commit();
@@ -831,12 +780,9 @@ not_free:
 				log_write_up_to(LSN_MAX, true);
 				DBUG_SUICIDE(););
 
-		for (ulint i = 0; i < TRX_SYS_N_RSEGS; ++i) {
-			if (trx_rseg_t* rseg = trx_sys.rseg_array[i]) {
-				ut_ad(rseg->is_persistent());
-				if (rseg->space == &space) {
-					rseg->skip_allocation = false;
-				}
+		for (auto& rseg : trx_sys.rseg_array) {
+			if (rseg.space == &space) {
+				rseg.clear_skip_allocation();
 			}
 		}
 
@@ -857,22 +803,22 @@ static void trx_purge_rseg_get_next_history_log(
 	trx_id_t	trx_no;
 	mtr_t		mtr;
 
-	mysql_mutex_lock(&purge_sys.rseg->mutex);
+	mtr.start();
+
+	purge_sys.rseg->latch.wr_lock();
 
 	ut_a(purge_sys.rseg->last_page_no != FIL_NULL);
 
-	purge_sys.tail.commit = purge_sys.rseg->last_commit + 1;
+	purge_sys.tail.trx_no = purge_sys.rseg->last_trx_no() + 1;
 	purge_sys.tail.undo_no = 0;
 	purge_sys.next_stored = false;
 
-	mtr.start();
-
 	const buf_block_t* undo_page = trx_undo_page_get_s_latched(
 		page_id_t(purge_sys.rseg->space->id,
 			  purge_sys.rseg->last_page_no), &mtr);
 
 	const trx_ulogf_t* log_hdr = undo_page->frame
-		+ purge_sys.rseg->last_offset;
+		+ purge_sys.rseg->last_offset();
 
 	/* Increase the purge page count by one for every handled log */
 
@@ -890,7 +836,7 @@ static void trx_purge_rseg_get_next_history_log(
 		purge_sys.rseg->last_page_no = FIL_NULL;
 	}
 
-	mysql_mutex_unlock(&purge_sys.rseg->mutex);
+	purge_sys.rseg->latch.wr_unlock();
 	mtr.commit();
 
 	if (empty) {
@@ -906,17 +852,19 @@ static void trx_purge_rseg_get_next_history_log(
 		+ prev_log_addr.boffset;
 
 	trx_no = mach_read_from_8(log_hdr + TRX_UNDO_TRX_NO);
-	unsigned purge = mach_read_from_2(log_hdr + TRX_UNDO_NEEDS_PURGE);
-	ut_ad(purge <= 1);
+	ut_ad(mach_read_from_2(log_hdr + TRX_UNDO_NEEDS_PURGE) <= 1);
 
 	mtr_commit(&mtr);
 
-	mysql_mutex_lock(&purge_sys.rseg->mutex);
+	purge_sys.rseg->latch.wr_lock();
 
 	purge_sys.rseg->last_page_no = prev_log_addr.page;
-	purge_sys.rseg->last_offset = prev_log_addr.boffset;
-	purge_sys.rseg->set_last_trx_no(trx_no, purge != 0);
-	purge_sys.rseg->needs_purge = purge != 0;
+	purge_sys.rseg->set_last_commit(prev_log_addr.boffset, trx_no);
+	if (log_hdr[TRX_UNDO_NEEDS_PURGE + 1]) {
+		purge_sys.rseg->set_needs_purge();
+	} else {
+		purge_sys.rseg->clear_needs_purge();
+	}
 
 	/* Purge can also produce events, however these are already ordered
 	in the rollback segment and any user generated event will be greater
@@ -929,22 +877,20 @@ static void trx_purge_rseg_get_next_history_log(
 
 	mysql_mutex_unlock(&purge_sys.pq_mutex);
 
-	mysql_mutex_unlock(&purge_sys.rseg->mutex);
+	purge_sys.rseg->latch.wr_unlock();
 }
 
 /** Position the purge sys "iterator" on the undo record to use for purging. */
-static
-void
-trx_purge_read_undo_rec()
+static void trx_purge_read_undo_rec()
 {
 	uint16_t	offset;
 	uint32_t	page_no;
 	ib_uint64_t	undo_no;
 
-	purge_sys.hdr_offset = purge_sys.rseg->last_offset;
+	purge_sys.hdr_offset = purge_sys.rseg->last_offset();
 	page_no = purge_sys.hdr_page_no = purge_sys.rseg->last_page_no;
 
-	if (purge_sys.rseg->needs_purge) {
+	if (purge_sys.rseg->needs_purge()) {
 		mtr_t		mtr;
 		mtr.start();
 		buf_block_t* undo_page;
@@ -1008,7 +954,7 @@ trx_purge_get_next_rec(
 	mtr_t		mtr;
 
 	ut_ad(purge_sys.next_stored);
-	ut_ad(purge_sys.tail.trx_no() < purge_sys.low_limit_no());
+	ut_ad(purge_sys.tail.trx_no < purge_sys.low_limit_no());
 
 	const ulint space = purge_sys.rseg->space->id;
 	const uint32_t page_no = purge_sys.page_no;
@@ -1098,7 +1044,7 @@ trx_purge_fetch_next_rec(
 		}
 	}
 
-	if (purge_sys.tail.trx_no() >= purge_sys.low_limit_no()) {
+	if (purge_sys.tail.trx_no >= purge_sys.low_limit_no()) {
 
 		return(NULL);
 	}
@@ -1110,7 +1056,7 @@ trx_purge_fetch_next_rec(
 		/* row_purge_record_func() will later set
 		ROLL_PTR_INSERT_FLAG for TRX_UNDO_INSERT_REC */
 		false,
-		purge_sys.rseg->id,
+		trx_sys.rseg_id(purge_sys.rseg, true),
 		purge_sys.page_no, purge_sys.offset);
 
 	/* The following call will advance the stored values of the
@@ -1244,7 +1190,7 @@ trx_purge_dml_delay(void)
 	/* If purge lag is set then calculate the new DML delay. */
 
 	if (srv_max_purge_lag > 0) {
-		double ratio = static_cast<double>(trx_sys.rseg_history_len) /
+		double ratio = static_cast<double>(trx_sys.history_size()) /
 			static_cast<double>(srv_max_purge_lag);
 
 		if (ratio > 1.0) {
